﻿//-----------------------------------------------------------------------
// <copyright file="AssemblyLoader.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using Logger;
    using Microsoft.Win32;
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Reflection;
    using System.Text;

    internal static class AssemblyLoader
    {
        private static List<string> assemblyLocations = new List<string>();

        static AssemblyLoader()
        {
            assemblyLocations.Add(GetTrfmDllsLocation());
            assemblyLocations.Add(GetBiztalkDllsPath());
            assemblyLocations.Add(GetBiztalkCommonDllsPath());
        }

        //// Read DLL locations from the registry
        internal static string GetTrfmDllsLocation()
        {
            try
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Wow6432Node\Microsoft\Windows Azure ServiceBus EAI & EDI Labs SDK\V1.5", false);
                return (string)key.GetValue("VsExtensionsDir");
            }
            catch (Exception e)
            {
                Controller.log.trace(MsgType.ERROR, "Unable to find Biztalk Services directory: " + e.Message);
                throw new Exception("Unable to find Biztalk Services directory");
            }
        }

        internal static string GetBiztalkDllsPath()
        {
            try
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Wow6432Node\Microsoft\BizTalk Server\3.0\XML Tools", false);
                return (string)key.GetValue("FunctoidAssemblyPath");
            }
            catch (Exception e)
            {
                Controller.log.trace(MsgType.ERROR, "Unable to find Biztalk Server directory: " + e.Message);
                throw new Exception("Unable to find Biztalk Server directory");
            }
        }

        internal static string GetBiztalkCommonDllsPath()
        {
            try
            {
                RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SYSTEM\ControlSet001\Control\Session Manager\Environment", false);
                return (string)key.GetValue("BTSCOMMONFILES");
            }
            catch (Exception e)
            {
                Controller.log.trace(MsgType.ERROR, "Unable to find Biztalk common files directory\nERROR: " + e.Message);
                throw new Exception("Unable to find common files directory");
            }
        }

        // This function tries to Resolve Biztalk and Trfm assemblies
        internal static Assembly Domain_AssemblyResolve(object sender, ResolveEventArgs args)
        {
            AssemblyName asmName = new AssemblyName(args.Name);
            Controller.log.debug(MsgType.INFO, "Received Resolve request for {0}" + asmName.Name);
            Assembly assembly = null;
            bool assemblyLoaded = false;

            try
            {
                string path = string.Empty;
                foreach (string assemblyLocation in assemblyLocations)
                {
                    path = Path.Combine(assemblyLocation, asmName.Name) + ".dll";
                    if (File.Exists(path))
                    {
                        assembly = Assembly.LoadFrom(path);
                        Controller.log.debug(MsgType.INFO, "Assembly Loaded {0}" + path);
                        assemblyLoaded = true;
                        break;
                    }
                }
                if(!assemblyLoaded)
                    throw new Exception();
            }
            catch (Exception e)
            {
                Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Unable to resolve Biztalk and TRFM assemblies\nERROR:" + e.Message);
                throw e;
            }

            return assembly;
        }

        internal static void AddAssemblyResolver()
        {
            AppDomain.CurrentDomain.AssemblyResolve += Domain_AssemblyResolve;
        }
    }
}
