﻿//-----------------------------------------------------------------------
// <copyright file="ConditionalAssignmentHandler.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using Logger;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;
  
    public class ConditionalAssignmentHandler : IXslElementHandler
    {
        ExpressionArgument condition;
        ExpressionArgument assignment;
        PathExpression target;

        // initalises the handler
        internal ConditionalAssignmentHandler(ExpressionArgument conditionValue, ExpressionArgument assignValue, PathExpression targetValue)
        {
            this.condition = conditionValue;
            this.assignment = assignValue;
            this.target = targetValue;
        }

        // converts the sub tree into a conditional assignment statement
        public object HandleElementConversion()
        {
            if (this.NullInputs())
            {
                return null;
            }

            Controller.log.trace(MsgType.INFO, "Condition:" + this.condition + " From:" + this.assignment + " To:" + this.target);

            return new ConditionalAssignment()
            {
                Condition = this.condition,
                From = this.assignment,
                To = this.target,
                Properties = 
                {
                    { "XCell", Constants.TRFMConstants.XcellValue },
                    { "YCell", Constants.TRFMConstants.YcellValue },
                    { "Label", string.Empty },
                    { "Id", TrfmCounters.GetNewFunctoidId() }
                }
            };
        }

        // Checks if any of the inputs to the conditional statement is null
        // if so, it return null and the conditional assignment statement is not created
        private bool NullInputs()
        {
            if (this.condition == null)
            {
                Controller.log.debug(MsgType.ERROR, "The conditon in the conditional assignment is null");
                return true;
            }

            if (this.assignment == null)
            {
                Controller.log.debug(MsgType.ERROR, "The assignment value in the conditional assignment is null");
                return true;
            }

            if (this.target == null)
            {
                Controller.log.debug(MsgType.ERROR, "The target node in the conditional assignment is null");
                return true;
            }

            return false;
        }
    }
}
