﻿//-----------------------------------------------------------------------
// <copyright file="Controller.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Xml;
    using System.Xml.Xsl;
    using BtmToTrfmConvertor.MapConvertor;
    using Logger;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;
    using Microsoft.BizTalk.BaseFunctoids;
    using Microsoft.BizTalk.Mapper.Compiler;
    using Microsoft.BizTalk.Mapper.OM;
    using Microsoft.BizTalk.TOM;
    using Microsoft.Win32;

    class Controller
    {
        public static Log log = null;

        static Controller()
        {
            try
            {
                log = Log.CreateInstance();
                AssemblyLoader.AddAssemblyResolver();
            }
            catch (Exception e)
            {
                Console.WriteLine("Initialization of the tool failed\nMake sure all the prerequisites are installed properly\nError: " + e.InnerException);
                Environment.Exit(-1);
            }
        }
        
        public static void Main(string[] arguments)
        {
            List<string> usageHelp = new List<string> { "/?", "?", "help" };
            if (arguments.Length > 2 || arguments.Length < 1 || usageHelp.Contains(arguments[0]))
            {
                Usage();
                return;
            }
            Controller.log.trace(MsgType.INFO, "The input to the tool - " + string.Join(",", arguments));

            try
            {
                ValidateArguments(arguments);
            }
            catch (Exception e)
            {
                Console.WriteLine("Input Validation Failed with exception: " + e.Message);
                return;
            }

            try
            {            
                // Change the working directory to BTM location so as to locate the schemas
                Environment.CurrentDirectory = Path.GetDirectoryName(arguments[0]);
                string outputFile = null;
                if (arguments.Length > 1)
                {
                    outputFile = arguments[1];
                }

                IMapConvertor mapConvertor = MapConvertorFactory.CreateMapConvertor(arguments[0], outputFile);
                mapConvertor.Convert();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Controller.log.debug(MsgType.ERROR, e.Message);
            }

            string path = Path.GetFullPath(arguments[0]);
            path = path.Replace(Path.GetFileName(arguments[0]), "Report.txt");
            Controller.log.CreateReport(Controller.log.LogPath, path);
        }      

        private static void ValidateArguments(string[] arguments)
        {
            // checking for the existence of the input file
            if (!File.Exists(arguments[0]))
            {
                log.debug(MsgType.ERROR, "BTM file is not found");
                log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "BTM file is not found");
                throw new FileNotFoundException(string.Format("Unable to find file \"{0}\". Try giving full path", arguments[0]));
            }
        }

        // Tool Usage
        private static void Usage()
        {
            Console.WriteLine("BtmToTrfmConvertor <BTMFilePath> [optional]<OutputTrfm>");
            Console.WriteLine("Example: BtmToTrfmConvertor SampleXsltDoc.btm Output.trfm");
        }
    }
}
