﻿// -----------------------------------------------------------------------
// <copyright file="ExpressionArgumentCreator.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using System.Collections.Generic;
    using Microsoft.ApplicationServer.Integration.Transforms.Common.Types;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;

    /// <summary>
    /// Creates Expression Arguments given a value
    ///     Pathexpression
    ///     ConstantExpression
    ///     ListPathExpression
    /// </summary>
    class ExpressionArgumentCreator
    {
        internal static PathExpression CreatePathExpression(string path)
        {
            if (string.IsNullOrEmpty(path))
            {
                path = string.Empty;
            }

            return new PathExpression
                {
                    Path = path
                };
        }

        internal static ConstantExpression CreateConstantExpression(string constantValue)
        {
            if (string.IsNullOrEmpty(constantValue))
            {
                constantValue = string.Empty;
            }

            return new ConstantExpression
                {
                    Value = constantValue
                };
        }

        internal static ListPathExpression CreateListPathExpression(string path)
        {
            if (string.IsNullOrEmpty(path))
            {
                path = string.Empty;
            }

            return new ListPathExpression
                {
                    Path = path
                };
        }

        internal static LambdaExpression CreateLambdaExpression(string expressionText, List<string> parameterNames, ITransformType returnType)
        {
            if (string.IsNullOrEmpty(expressionText))
            {
                expressionText = string.Empty;
            }

            if (parameterNames == null)
            {
                parameterNames = new List<string>();
            }

            if (returnType == null)
            {
                returnType = SimpleType.String;
            }

            LambdaExpression lambdaExpression = new LambdaExpression
            {
                ExpressionText = expressionText,
                ReturnType = returnType,
                Properties =
                {
                     { "SettingName", "Selector" }
                },
            };

            foreach (string parameterName in parameterNames)
            {
                lambdaExpression.ParameterNames.Add(parameterName);
            }

            return lambdaExpression;
        }
    }
}
