﻿//-----------------------------------------------------------------------
// <copyright file="FunctoidCreation.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
 
namespace BtmToTrfmConvertor
{
    using Logger;

    class FunctoidCreation
    {
        string targetFunction;
        string selectValue;
        OperationContext context;
        string content;

        // initialises the class
        public FunctoidCreation(string target, string selectValue, OperationContext operationContext, string content = null)
        {
            this.targetFunction = target;
            this.selectValue = selectValue;
            this.context = operationContext;
            this.content = content;
        }

        // calls the correct INodeVisitable node based on the input functoid call
        // returns the trfm object created to be added to the VariableInitialization part of the parent object
        internal object HandleConversion()
        {
            INodeVisitable functoidCreated;
            Types.FunctoidType type = (Types.FunctoidType)this.targetFunction[0];
            switch (type)
            {
                case Types.FunctoidType.StringFunctoid:
                    functoidCreated = new FunctoidInvocationCreator(this.targetFunction.Substring(1), this.selectValue, this.context);
                    break;
                case Types.FunctoidType.ArithmeticExpression:
                    functoidCreated = new ArithmeticExpressionCreator(this.targetFunction.Substring(1), this.selectValue, this.context);
                    break;
                case Types.FunctoidType.LogicalExpression:
                    functoidCreated = new LogicalExpressionCreator(this.targetFunction.Substring(1), this.selectValue, this.context);
                    break;
                case Types.FunctoidType.DateTimeFunctoid:
                    functoidCreated = new FunctoidInvocationCreator(this.targetFunction.Substring(2), "(\"" + this.GetDateTimeInput((Types.DateTimeFunctoidType)this.targetFunction[1]) + "\")", this.context);
                    break;
                case Types.FunctoidType.CumulativeFunctoid:
                    functoidCreated = new FunctoidInvocationCreator(this.targetFunction.Substring(1), this.GetCumulativeInput(), this.context);
                    break;
                case Types.FunctoidType.ScriptingFunctoid:
                    functoidCreated = new ScriptInvocationCreator(this.targetFunction.Substring(1), this.selectValue, this.context, this.content);
                    break;
                default:
                    functoidCreated = null;
                    break;
            }

            if (functoidCreated != null)
            {
                INodeVisitor v = new NodeVisitor();
                return functoidCreated.Accept(v);
            }
            else
            {
                Controller.log.debug(MsgType.ERROR, "functoid present in dictionary as supported but not handled");
                return null;
            }
        }

        // returns the input node on which the cumulation is done
        private string GetCumulativeInput()
        {
            string input;
            if (this.context.CurrentListInitialisation != null)
            {
                input = this.context.CurrentListInitialisation.Identifier;
            }
            else
            {
                input = Constants.CumulativeInput;
            }

            return "Cumulative(" + input + ")";
        }

        // returns the input format of the datetime generate functoid in TRFM based on the DateTime functoid in BTM
        private string GetDateTimeInput(Types.DateTimeFunctoidType dateTimeType)
        {
            string input = string.Empty;
            switch (dateTimeType)
            {
                case Types.DateTimeFunctoidType.DateCurrentDate:
                    input = "yyyy-MM-dd";
                    break;
                case Types.DateTimeFunctoidType.DateCurrentTime:
                    input = "HH:MM:SS";
                    break;
                case Types.DateTimeFunctoidType.DateCurrentDateTime:
                    input = "yyyy-MM-ddThh:mm:ss";
                    break;
            }

            return input;
        }
    }
}
