﻿//-----------------------------------------------------------------------
// <copyright file="ListCreator.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using System.Collections.Generic;
    using Logger;
    using Microsoft.ApplicationServer.Integration.Transforms.Common.Types;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;

    internal class ListCreator
    {
        List<MemberDetails> members;
        ListInitialization listInit;
        OperationContext context;

        internal ListCreator(OperationContext oc)
        {  
            this.context = oc;
            this.listInit = new ListInitialization()
            {
                Initializer = new Block(),
                Identifier = Constants.TRFMConstants.VariableTag + ++TrfmCounters.VariableCounter,
                Properties =
                    {
                        { "XCell", Constants.TRFMConstants.XcellValue },
                        { "YCell", Constants.TRFMConstants.YcellValue },
                        { "Label", string.Empty },
                        { "Id", TrfmCounters.GetNewFunctoidId() }
                    }
            };
            this.members = new List<MemberDetails>();
        }

        internal ListCreator(ListInitialization inputList, OperationContext oc)
        {
            this.listInit = inputList;
            this.context = oc;
        }

        internal ListInitialization ListInitializationNode
        {
            get { return this.listInit; }
        }

        internal AddItemToList AddItems(List<string> inputs, List<ITransformType> types)
        {
            int index = 0;
            MemberDetails member;
            foreach (string input in inputs)
            {
                if (types.Count <= index)
                {
                    types.Add(SimpleType.String); // adding string as it the default value
                }

                if (this.members == null)
                {
                    this.members = new List<MemberDetails>();
                }

                member = new MemberDetails 
                { 
                    Input = PathCreation.CreateSourceNode(input, this.context), 
                    InputType = types[index], 
                    ColumnName = "Input_" + this.members.Count 
                };
                this.members.Add(member);
                index++;
            }

            this.AssignMembers();
            return this.CreateAddItem();
        }

        internal void AddObjectToList(object listContent)
        {
            if (listContent.GetType() == typeof(VariableInitialization))
            {
                this.listInit.Initializer.Statements.Add((VariableInitialization)listContent);
            }
            else if (listContent.GetType() == typeof(Foreach))
            {
                this.listInit.Initializer.Statements.Add((Foreach)listContent);
            }
            else if (listContent.GetType() == typeof(Mapeach))
            {
                this.listInit.Initializer.Statements.Add((Mapeach)listContent);
            }
            else
            {
                Controller.log.debug(MsgType.ERROR, "Unable to add object of type " + listContent.GetType().ToString() + " to the list " + this.listInit.Identifier);
            }
        }

        #region private functions

        private void AssignMembers()
        {
            List<MemberInfo> members = new List<MemberInfo>();
            MemberInfo member;
            foreach (MemberDetails detail in this.members)
            {
                member = new MemberInfo(detail.ColumnName, detail.InputType);
                members.Add(member);
            }

            NativeTransformType newTransformType = TypeFactory.GetNativeTransformType(members);
            this.listInit.Type = newTransformType;
        }

        // creates an AddItemToList functoid
        private AddItemToList CreateAddItem()
        {
            AddItemToList addItemToList = new AddItemToList();

            // not considering the conditonal expression
            foreach (MemberDetails member in this.members)
            { 
                addItemToList.Values.Add(new NameBinding()
                {
                    Expression = this.CreateInputPath(member.Input),
                    Name = member.ColumnName,
                    Type = member.InputType
                });
            }

            return addItemToList;
        }

        private ExpressionArgument CreateInputPath(string input)
        {
            if (input.StartsWith("\"") && input.EndsWith("\""))
            {
                return ExpressionArgumentCreator.CreateConstantExpression(input.Substring(1, input.Length - 2));
            }
            else
            {
                return ExpressionArgumentCreator.CreatePathExpression(input);
            }
        }

        #endregion

        struct MemberDetails
        {
            internal string Input;
            internal ITransformType InputType;
            internal string ColumnName;
        }
    }
}
