﻿//-----------------------------------------------------------------------
// <copyright file="DesignMapConvertor.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor.MapConvertor
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Xml;
    using System.Xml.Xsl;
    using BtmToTrfmConvertor;
    using Logger;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;

    public class DesignMapConvertor : MapConvertor
    {
        internal static string SourceSchemaPath = string.Empty;
        internal static string TargetSchemaPath = string.Empty;

        private static string xsltPath;

        internal DesignMapConvertor(string inputBtmFile, string outputTrfmFile) : base(inputBtmFile, outputTrfmFile) {}

        // Properties
        public static string XsltPath
        {
            get { return xsltPath; }
        }

        public override void Convert()
        {
            // Replace Inline XSLT script code with C# Script and generate xsl
            xsltPath = XsltGenerator.GenerateCustomisedXSLT(inputFile);

            // Validate XSLT
            Controller.log.trace(MsgType.INFO, "Validating Xsl");
            if (ValidateXsl(xsltPath))
            {
                MapFile file = base.GetTrfmBase();
                BtmToTrfmConvertor.Constants.map1 = file.Maplets[0];
                file.MainMapletName = "Page 1"; // call
                BtmToTrfmConvertor.Constants.map1.Name = "Page 1"; // call
                
                using (XmlTextReader reader = new XmlTextReader(XsltPath))
                {
                    XslReader xslreader = new XslReader(reader);
                    Controller.log.trace(MsgType.INFO, "Trying to parse the xsl");
                    xslreader.Parse();

                    AddPrefixesToMapFile(file);
                    file.Save(outputFile);
                    Controller.log.trace(MsgType.INFO, "Output trfm saved at " + outputFile);
                }

            }
            
            if (File.Exists(xsltPath))
            {
                File.Delete(xsltPath);
            }
        }

        private void AddPrefixesToMapFile(MapFile mapFile)
        {
            Controller.log.trace(MsgType.INFO, "Generating namespace Prefix");
            foreach (KeyValuePair<string, string> kvp in BtmToTrfmConvertor.Constants.PrefixDictionary)
            {
                mapFile.XmlNamespacePrefixMap.Add(kvp.Value, BtmToTrfmConvertor.Constants.NamespacePrefix[kvp.Key]);
            }

            if (!string.IsNullOrEmpty(BtmToTrfmConvertor.Constants.sourceNameSpace) && !mapFile.XmlNamespacePrefixMap.ContainsValue(BtmToTrfmConvertor.Constants.sourceNameSpace))
            {
                mapFile.XmlNamespacePrefixMap.Add("ns" + BtmToTrfmConvertor.Constants.PrefixDictionary.Count, BtmToTrfmConvertor.Constants.sourceNameSpace);
            }

            if (!string.IsNullOrEmpty(BtmToTrfmConvertor.Constants.targetNameSpace) && !mapFile.XmlNamespacePrefixMap.ContainsValue(BtmToTrfmConvertor.Constants.targetNameSpace))
            {
                mapFile.XmlNamespacePrefixMap.Add("ns" + BtmToTrfmConvertor.Constants.PrefixDictionary.Count + 1, BtmToTrfmConvertor.Constants.targetNameSpace);
            }
        }

        // This function checks if an XSL is well formed.
        // We'll support only XSL 1.0 spec as supported by .Net
        // TBD: Validate schema files
        private static bool ValidateXsl(string xsltFilePath)
        {
            try
            {
                XslTransform xslt = new XslTransform();
                xslt.Load(xsltFilePath); // Load the stylesheet.
                Controller.log.trace(MsgType.INFO, "Valid Xsl");
                return true;
            }
            catch (Exception e)
            {
                Console.WriteLine("Not a well formed XML");
                Controller.log.debug(MsgType.ERROR, "Not a well formed XML..Stack Trace\n" + e.StackTrace);
                Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Not a well formed XML..Stack Trace\n" + e.StackTrace);
                return false;
            }
        }
    }
}
