﻿//-----------------------------------------------------------------------
// <copyright file="MapConvertor.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor.MapConvertor
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Text;
    using Logger;
    using Microsoft.BizTalk.Mapper.OM;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;

    public abstract class MapConvertor : IMapConvertor
    {
        protected string inputFile, outputFile;

        internal MapConvertor(string inputBtmFile, string outputTrfmFile)
        {
            inputFile = inputBtmFile;
            outputFile = outputTrfmFile;
        }

        public abstract void Convert();

        internal MapFile GetTrfmBase()
        {
            Map map = new Map();
            map.LoadFunctoids(null);
            map.Load(inputFile);

            MapFile file = new MapFile();
            Maplet maplet;

            if (this is CustomXsltMapConvertor)
            {
                CustomXsltMapConvertor.ExtractData(map);
                maplet = new XsltBasedMaplet();
            }
            else
            {
                maplet = new Maplet();
            }

            // Get Parameter Info for source
            ParameterInfo srcParam = new ParameterInfo();
            srcParam.Name = BtmToTrfmConvertor.Constants.TRFMConstants.ScopeSourceHome;
            XmlEntityType srcXmlEntityType = new XmlEntityType()
            {
                LocalName = map.SourceTree.GetRootNodeName(),
                NamespaceName = map.SourceTree.TargetNamespace,
                SchemaLocation = Path.GetFullPath(map.SourceTreePath)
            };
            srcParam.Type = srcXmlEntityType;

            // Get Parameter Info for target
            ParameterInfo tgtParam = new ParameterInfo();
            tgtParam.Name = BtmToTrfmConvertor.Constants.TRFMConstants.ScopeTargetHome;
            XmlEntityType tgtXmlEntityType = new XmlEntityType()
            {
                LocalName = map.TargetTree.GetRootNodeName(),
                NamespaceName = map.TargetTree.TargetNamespace,
                SchemaLocation = Path.GetFullPath(map.TargetTreePath)
            };
            tgtParam.Type = tgtXmlEntityType;

            maplet.SourceParameters.Add(srcParam);
            maplet.TargetParameters.Add(tgtParam);
            Controller.log.trace(MsgType.INFO, "Schema Info updated");


            file.Maplets.Add(maplet);
            BtmToTrfmConvertor.Constants.sourceNameSpace = map.SourceTree.TargetNamespace;
            BtmToTrfmConvertor.Constants.targetNameSpace = map.TargetTree.TargetNamespace;
            
            return file;
        }
    }
}
