﻿//-----------------------------------------------------------------------
// <copyright file="MapConvertorFactory.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor.MapConvertor
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Text;
    using Logger;
    using Microsoft.BizTalk.Mapper.OM;

    public static class MapConvertorFactory
    {
        public static IMapConvertor CreateMapConvertor(string inputBtmFile, string outputTrfmFile)
        {
            outputTrfmFile = GetOutputFile(inputBtmFile, outputTrfmFile);

            Controller.log.trace(MsgType.INFO, "output trfm path assigned" + outputTrfmFile);
            Map map = new Map();
            map.LoadFunctoids(null);
            map.Load(inputBtmFile);

            if (map.CustomXsltPath != null)
            {
                return new CustomXsltMapConvertor(inputBtmFile, outputTrfmFile);
            }
            else
            {
                return new DesignMapConvertor(inputBtmFile, outputTrfmFile);
            }
        }

        private static string GetOutputFile(string inputBtmFile, string outputTrfmFile)
        {
            if (outputTrfmFile == null)
            {
                if (string.IsNullOrEmpty(Path.GetExtension(inputBtmFile)))
                {
                    outputTrfmFile = inputBtmFile + ".trfm";
                }
                else if (Path.GetExtension(inputBtmFile).ToLower().Equals(".btm"))
                {
                    outputTrfmFile = inputBtmFile.Substring(0, inputBtmFile.Length - 4) + ".trfm";
                }
                else
                {
                    Controller.log.debug(MsgType.ERROR, "Input btm file is invalid");
                    throw new Exception("Input btm file is invalid");
                }
            }

            return outputTrfmFile;
        }

    }
}
