﻿//-----------------------------------------------------------------------
// <copyright file="OperationContext.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using System.Collections.Generic;
    using System.Linq;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;

    public class OperationContext
    {
        Dictionary<string, string> varDictionary = new Dictionary<string, string>(); // holds the xsl variable declared as key and the trfm equivalent path as value
        List<object> itemsInCurrentContext;
        Dictionary<string, string> scopeAndLoopVariables = new Dictionary<string, string>(); // holds the scopeLabels and their respective sourceloopvariable paths 

        // intialises the class
        public OperationContext()
        {
            Scope = Constants.TRFMConstants.ScopeHome;
            TargetScope = string.Empty;
            SubTreePath = string.Empty; 
            this.itemsInCurrentContext = new List<object>();
            CurrentListInitialisation = null;
        }

        // copy constructor
        public OperationContext(OperationContext oc)
        {
            Scope = oc.Scope;
            TargetScope = oc.TargetScope;
            SubTreePath = oc.SubTreePath;
            this.varDictionary = new Dictionary<string, string>();
            this.itemsInCurrentContext = new List<object>();
            CurrentListInitialisation = oc.CurrentListInitialisation;
            foreach (KeyValuePair<string, string> entry in oc.varDictionary)
            {
                this.varDictionary.Add(entry.Key, entry.Value);
            }

            foreach (KeyValuePair<string, string> entry in oc.scopeAndLoopVariables)
            {
                this.scopeAndLoopVariables.Add(entry.Key, entry.Value);
            }
        }

        // properties
        internal ListInitialization CurrentListInitialisation { get; set; }

        internal string Scope { get; set; }

        internal string SubTreePath { get; set; } // the sub tree path in case of foreach loop

        internal string TargetScope { get; set; }

        // get and set dictionary value
        internal string GetVarDictionary(string key) 
        {
            return this.varDictionary[key];
        }

        internal void SetVarDictionary(string key, string value)
        {
            this.varDictionary.Add(key, value);
        }

        // contains value?
        internal bool ContainsKey(string key)
        {
            return this.varDictionary.ContainsKey(key);
        }

        // items to be added to the current context before exiting te context
        internal void AddItemToCurrentContext(object item)
        {
            this.itemsInCurrentContext.Add(item);
        }

        internal List<object> GetItemsInCurrentContext()
        {
            return this.itemsInCurrentContext;
        }

        internal void RemoveItemFromCurrentContext(int index)
        {
            if (index < this.itemsInCurrentContext.Count())
            {
                this.itemsInCurrentContext.RemoveAt(index);
            }
        }

        // set and remove dictionary values for scope loop variables
        internal void SetScopeAndLoopVariables(string key, string value)
        {
            if (this.scopeAndLoopVariables.ContainsKey(key))
            {
                this.scopeAndLoopVariables.Remove(key);
            }

            this.scopeAndLoopVariables.Add(key, value);
        }

        internal void RemoveScopeAndLoopVariablesWithValue(string value)
        {
            if (this.scopeAndLoopVariables.ContainsValue(value))
            {
                foreach (KeyValuePair<string, string> entry in this.scopeAndLoopVariables)
                {
                    if (entry.Value.Equals(value))
                    {
                        this.scopeAndLoopVariables.Remove(entry.Key);
                    }
                }
            }
        }

        internal void ReplaceScopeAndLoopVariables(string oldKey, string newKey)
        {
            this.scopeAndLoopVariables.Add(newKey, this.scopeAndLoopVariables[oldKey]);
            this.scopeAndLoopVariables.Remove(oldKey);
        }

        internal string GetScopeAndLoopVariables(string key)
        {
            return this.scopeAndLoopVariables[key];
        }
    }
}
