﻿//-----------------------------------------------------------------------
// <copyright file="Optimiser.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Text.RegularExpressions;
    using Microsoft.ApplicationServer.Integration.Transforms.Common.Types;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;
    using Logger;

    public class Optimiser
    {
        internal static void SetType(ref LogicalExpression expression)
        {
            string expressionText = expression.ExpressionText;
            
                // TODO : change this to properly parse and calc the type
            if (expressionText.Contains(">") || expressionText.Contains("<"))
            {
                for (int i = 0; i < expression.Aliases.Count; i++)
                {
                    expression.Aliases[i].Type = SimpleType.Number;
                }
            }
            else if (expressionText.Contains("!=") || expressionText.Contains("=="))
            {
                if (expression.Aliases[0].Type != SimpleType.String)
                {
                    expression.Aliases[1].Type = expression.Aliases[0].Type;
                }
                else
                {
                    expression.Aliases[0].Type = expression.Aliases[1].Type;
                }
            }
            else if (expressionText.Contains("!") || expressionText.Contains("&&") || expressionText.Contains("||"))
            {
                for (int i = 0; i < expression.Aliases.Count; i++)
                {
                    expression.Aliases[i].Type = SimpleType.Boolean;
                }
            }
        }

        internal static PathExpression redundantLogicalExpression(LogicalExpression expression, OperationContext context)
        {
            // check there are only 2 aliases 
            if (expression.Aliases.Count == 2)
            {
                // check that the aliases are one constant expression and one logical expression
                if (expression.Aliases[0].Expression is PathExpression && expression.Aliases[1].Expression is ConstantExpression)
                {
                    // check that the path expression points to logical expression 
                    PathExpression path = expression.Aliases[0].Expression as PathExpression;
                    bool isLogicalExpression = false;
                    List<object> statements = context.GetItemsInCurrentContext();
                    foreach (object statement in statements)
                    {
                        if (statement is VariableInitialization)
                        {
                            VariableInitialization variable = statement as VariableInitialization;
                            if (variable.Initializer is LogicalExpression && variable.Identifier == path.Path)
                            {
                                isLogicalExpression = true;
                                break;
                            }
                        }
                    }

                    // check for constant expression
                    if (isLogicalExpression)
                    {
                        ConstantExpression constant = expression.Aliases[1].Expression as ConstantExpression;
                        string constValue = constant.Value.ToString().ToLower();
                        if (constValue.Equals("true") || constValue.Equals("false"))
                        {
                            // check the expression
                            string expressionValue = expression.Aliases[0].Name + "==" + expression.Aliases[1].Name;
                            if (expression.ExpressionText == expressionValue)
                            {
                                return path;
                            }
                        }
                    }
                }
            }
            return null;
        }
    }
}
