﻿//-----------------------------------------------------------------------
// <copyright file="XslExpressionHandler.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using System;
    using System.Collections.Generic;
    using System.Text.RegularExpressions;
    using Logger;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;
    using Microsoft.ApplicationServer.Integration.Transforms.Common.Types;
 
    public class XslExpressionHandler
    {
        OperationContext context;

        public XslExpressionHandler()
        {
        }

        public XslExpressionHandler(OperationContext operationContext)
        {
            this.context = operationContext;
        }

        enum ExpressionType
        {
            LogicalExpression = 1,
            ArithmeticExpression
        }

        internal object ParseExpression(string value)
        {
            Controller.log.trace(MsgType.INFO, "input for parsing :" + value);
            
            Regex pattern;
            MatchCollection matches;

            value = value.Trim();

            // for input 0 in some functoids such as dblookup and cumulative
            if (value.Equals("0"))
            {
                Controller.log.debug(MsgType.ERROR, "input is 0");
                return null;
            }

            if (string.IsNullOrEmpty(value))
            {
                Controller.log.trace(MsgType.INFO, "input is empty string");
                return this.CreateExpressionArgument(string.Empty, false);
            }

            // ---------------------------------------Constant expresion
            if ((value.StartsWith("\"") && value.EndsWith("\"")) || (value.Trim().StartsWith("'") && value.Trim().EndsWith("'")))
            {
                int instanceCount1 = value.Split('"').Length - 1;
                int instanceCount2 = value.Split('\'').Length - 1;

                // in case there are more than 2 " or ', one at the start and end each,
                // then check if it is an expression before creating a constant 
                if (instanceCount1 == 2 || instanceCount2 == 2)
                {
                    Controller.log.trace(MsgType.INFO, "input is a constant expression");
                    return this.CreateExpressionArgument(value, false);
                }
            }

            // --------------------------------------Logical expression and binary operators(without not!)
            pattern = new Regex(@">|=|<|!|\||&");
            matches = pattern.Matches(value);
            if (matches.Count > 0)
            {
                Controller.log.trace(MsgType.INFO, "input is a logical expression");
                return this.CreateExpression(value, matches, ExpressionType.LogicalExpression);
            }

            // -----------------------------------------Arithmetic Operators(witout division)
            pattern = new Regex(@"\*|\+|-|%|\\");
            matches = pattern.Matches(value);
            if (matches.Count > 0)
            {
                Controller.log.trace(MsgType.INFO, "input is an arithmetic expression");
                return this.CreateExpression(value, matches, ExpressionType.ArithmeticExpression);
            }

            // -----------------------------------------------Check for functions
            int startIndex = value.IndexOf("(");
            if (startIndex != -1 && value[value.Length - 1] == ')')
            {
                Controller.log.trace(MsgType.INFO, "input has functions");
                if ((startIndex + 1) == (value.Length - 1))
                {
                    if (startIndex > 3 && value.Substring(startIndex - 4, 4).Equals("text", StringComparison.OrdinalIgnoreCase))
                    {
                        // decode xpath
                        Controller.log.trace(MsgType.INFO, "Parsing:function text() encountered");
                        return this.CreateExpressionArgument(value, true);
                    }
                    else if (startIndex > 3 && value.Substring(startIndex - 4, 4).Equals("true", StringComparison.OrdinalIgnoreCase))
                    {
                        Controller.log.trace(MsgType.INFO, "Parsing:function true() encountered");
                        return this.CreateExpressionArgument("true", false);
                    }
                    else if (startIndex > 3 && value.Substring(startIndex - 4, 4).Equals("false", StringComparison.OrdinalIgnoreCase))
                    {
                        Controller.log.trace(MsgType.INFO, "Parsing:function false() encountered");
                        return this.CreateExpressionArgument("false", false);
                    }
                    else if (startIndex > 7 && value.Substring(startIndex - 8, 8).Equals("position", StringComparison.OrdinalIgnoreCase))
                    {
                        // handle position function
                        VariableHandler handler = new VariableHandler(this.context);
                        VariableInitialization variableNode = handler.AddFunctoid("Iteration", "()", false);
                        Controller.log.trace(MsgType.INFO, "Parsing:GetCurrentLoopIndex functoid created");
                        return variableNode;
                    }
                    else
                    {
                        Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Could not resolve link path");
                        return null;
                    }
                }
                else
                {
                    // input has functions which take inputs themselves
                    if (startIndex > 5 && value.Substring(startIndex - 6, 6).Equals("string", StringComparison.OrdinalIgnoreCase))
                    {
                        Controller.log.trace(MsgType.INFO, "function string() encountered");
                        return this.CreateExpressionArgument(value.Substring(startIndex + 1, value.Length - startIndex - 2), true);
                    }
                    else if (startIndex > 4 && value.Substring(startIndex - 5, 5).Equals("count", StringComparison.OrdinalIgnoreCase))
                    {
                        // handle count function
                        VariableHandler handler = new VariableHandler(this.context);
                        Constants.CumulativeInput = value.Substring(startIndex + 1, value.Length - startIndex - 2);
                        if (Constants.CumulativeInput.StartsWith(Constants.sourceTemplate))
                        {
                            Constants.CumulativeInput = Constants.CumulativeInput.Substring(Constants.sourceTemplate.Length);
                        }

                        VariableInitialization variableNode = handler.AddFunctoid("count", Constants.CumulativeInput, false);
                        if (variableNode != null)
                        {
                            FunctionInvocationExpression functoid = variableNode.Initializer as FunctionInvocationExpression;
                            if (functoid != null)
                            {
                                if (functoid.Target is PathExpression)
                                {
                                    string path = ((PathExpression)functoid.Target).Path;
                                    functoid.Target = new ListPathExpression { Path = path };
                                }
                            }
                        }

                        Constants.map1.Statements.Add(variableNode);
                        variableNode = handler.CreateConstantFunctoid(variableNode.Identifier, false);
                        Controller.log.trace(MsgType.INFO, "Cumulative count functoid created");
                        return variableNode;
                    }
                    else if (startIndex > 6 && value.Substring(startIndex - 7, 7).Equals("boolean", StringComparison.OrdinalIgnoreCase))
                    {
                        return this.CreateExpressionArgument(value.Substring(startIndex + 1, value.Length - startIndex - 2), true);
                    }
                    else
                    {
                        Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Could not resolve link path");
                        return null;
                    }
                }
            }

            // ------------------------------------------------Variables
            if (value.StartsWith("$var:"))
            {
                Controller.log.trace(MsgType.INFO, "variable encountered");
                return this.CreateExpressionArgument(value, true);
            }
            else if (value.StartsWith(Constants.TRFMConstants.VariableTag))
            {
                Controller.log.trace(MsgType.INFO, "trfm variable encountered");
                return ExpressionArgumentCreator.CreatePathExpression(value);
            }

            Controller.log.trace(MsgType.INFO, "path expression created");
            return this.CreateExpressionArgument(value, true);
        }

        internal PathExpression CreateNullPath()
        {
            Controller.log.debug(MsgType.ERROR, "null path epression created");
            Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Unable to convert Input link");
            return ExpressionArgumentCreator.CreatePathExpression(null);
        }

        internal NameBinding formatInput(object input, string alias, ITransformType type = null)
        {
            Controller.log.trace(MsgType.INFO, "namebinding to inputs started");
            ExpressionArgument inputPath = null;
            if(type == null)
            {
                type = getInputType(input);
            }
            // adding the inputs to the logical expression

            if (input == null)
            {
                input = this.CreateNullPath();
            }

            if (input is ConstantExpression)
            {
                inputPath = input as ConstantExpression;
            }
            else if (input is PathExpression)
            {
                inputPath = input as PathExpression;
            }
            else if (input is VariableInitialization)
            {
                // get the path expression using the variable name
                inputPath = ExpressionArgumentCreator.CreatePathExpression(((VariableInitialization) input).Identifier);
            }
            else
            {
                inputPath = null; // ERROR as none of the expression types were returned
            }

            Controller.log.trace(MsgType.INFO, string.Format("Created input namebinding Alias:{0} Type:", alias));
            return new NameBinding()
                       {
                           Expression = inputPath,
                           Name = alias,
                           Type = type
                       };

        }

        internal ExpressionArgument CreateExpressionArgument(string value, bool isPathExpression)
        {
            Controller.log.trace(MsgType.INFO, "creating expression argument");
            if (isPathExpression)
            {
                Controller.log.trace(MsgType.INFO, "path expression being created");
                string path = PathCreation.CreateSourceNode(value, this.context);
                if (path == null)
                {
                    return this.CreateNullPath();
                }

                path = path.Trim();
                if (path.StartsWith("\"") && path.EndsWith("\""))
                {
                    value = path;
                }
                else
                {
                    return ExpressionArgumentCreator.CreatePathExpression(path);
                }
            }

            Controller.log.trace(MsgType.INFO, "constant expression being created");
            if (string.IsNullOrEmpty(value))
            {
                return ExpressionArgumentCreator.CreateConstantExpression(string.Empty);
            }
            else
            {
                return ExpressionArgumentCreator.CreateConstantExpression(value.Substring(1, value.Length - 2));
            }
        }

        private List<char> GenerateSymbols(ExpressionType expressionType)
        {
            switch (expressionType)
            {
                case ExpressionType.LogicalExpression:
                    return new List<char>()
                        {
                            '>',
                            '<',
                            '!',
                            '&',
                            '=',
                            '|'
                        };
  
                case ExpressionType.ArithmeticExpression:
                    return new List<char>()
                        {
                            '+',
                            '-',
                            '\\',
                            '*',
                            '%'
                        };
                default:
                    return null;
            }  
        }

        private object CreateExpression(string value, MatchCollection matches, ExpressionType expressionType)
        {
            Controller.log.trace(MsgType.INFO, "creating inputs and expression started");
            int count = matches.Count;
            int index = 1;
            string childExpression = string.Empty,alias;
            object input;
            List<NameBinding> inputs = new List<NameBinding>();
            int startIndex = 0, currentIndex, j;
            string expression = string.Empty;
            List<char> symbols = this.GenerateSymbols(expressionType);
            for (int i = 0; i < count; i++)
            {
                alias = "Input_" + index;
                currentIndex = matches[i].Index;
                childExpression = value.Substring(startIndex, currentIndex - startIndex);
                startIndex = currentIndex + 1;
                if (childExpression.Equals(string.Empty))
                {
                    expression = expression + value.Substring(currentIndex, 1);
                    continue;
                }

                input = this.ParseExpression(childExpression);
                if (input == null)
                {
                    input = this.CreateNullPath();
                }
                else if (input is PathExpression)
                {
                    // nullable node handling
                    PathExpression temp = (PathExpression) input;
                    if (temp.Path.StartsWith("nil:"))
                    {
                        return input;
                    }
                }

                inputs.Add(this.formatInput(input,alias));
                int symbolCount = symbols.Count;
                for (j = 0; j < symbolCount; j++)
                {
                    if (value[startIndex] == symbols[j])
                    {
                        break;
                    }
                }

                if (j != symbolCount)
                {
                    startIndex++;
                    expression = expression + alias + value.Substring(currentIndex, 2);
                    i++;
                }
                else
                {
                    expression = expression + alias + value.Substring(currentIndex, 1);
                    if (value[currentIndex] == '=')
                    {
                        // for the test attribute == is not present but = is present
                        expression += "=";
                    }
                }

                index++;
            }

            input = this.ParseExpression(value.Substring(startIndex, value.Length - startIndex));
            if (input == null)
            {
                input = this.CreateNullPath();
            }

            alias = "Input_" + index;

            inputs.Add(this.formatInput(input,alias));
            if (expressionType == ExpressionType.ArithmeticExpression)
            {
                // all inputs must be of type number
                for (int temp = 0; temp < inputs.Count; temp++)
                {
                    inputs[temp].Type = SimpleType.Number;
                }
            }

            expression = expression + alias;
            expression = expression.Replace('\\', '/');
            return this.CreateVariableInitialization(expressionType, expression, inputs);
        }

        private VariableInitialization CreateVariableInitialization(ExpressionType choice, string expression, List<NameBinding> inputs)
        {
            Controller.log.trace(MsgType.INFO, "creating a variable");
            int inputCount;
            VariableInitialization variableNode = VariableHandler.InitializeVariableInitialization(null);
            switch (choice)
            {
                case ExpressionType.LogicalExpression:
                    LogicalExpression logicalExpression = new LogicalExpression();
                    logicalExpression.ExpressionText = expression;
                    variableNode.Initializer = logicalExpression;
                    inputCount = inputs.Count;
                    for (int j = 0; j < inputCount; j++)
                    {
                        logicalExpression.Aliases.Add(inputs[j]);
                    }

                    Controller.log.trace(MsgType.INFO, "creating a logical expression:" + logicalExpression.ExpressionText);
                    break;
                case ExpressionType.ArithmeticExpression:
                    ArithmeticExpression arithmeticExpression = new ArithmeticExpression();
                    arithmeticExpression.ExpressionText = expression;
                    variableNode.Initializer = arithmeticExpression;
                    inputCount = inputs.Count;
                    for (int j = 0; j < inputCount; j++)
                    {
                        arithmeticExpression.Aliases.Add(inputs[j]);
                    }

                    Controller.log.trace(MsgType.INFO, "creating a arithmetic expression" + arithmeticExpression.ExpressionText);
                    break;
                default:
                    Controller.log.debug(MsgType.ERROR, "not a logical or arithmetic expression");
                    return null;
            }

            Controller.log.trace(MsgType.INFO, "variable created");
            return variableNode;
        }

        private ITransformType getInputType(object input)
        {
            if (input == null)
            {
                return SimpleType.String;
            }

            if (input is ConstantExpression)
            {
                string constantValue = ((ConstantExpression) input).Value.ToString();
                try
                {
                    double temp1;
                    Boolean temp2;
                    if (double.TryParse(constantValue, out temp1))
                    {
                        return SimpleType.Number;
                    }
                    else if (Boolean.TryParse(constantValue, out temp2))
                    {
                        return SimpleType.Boolean;
                    }
                }
                catch (Exception)
                {
                }
            }
            else if (input is PathExpression)
            {
                string source = ((PathExpression) input).Path;
                List<object> statements = this.context.GetItemsInCurrentContext();
                foreach (object statement in statements)
                {
                    if (statement is VariableInitialization)
                    {
                        VariableInitialization variable = statement as VariableInitialization;
                        if (variable.Identifier == source)
                        {
                            if (variable.Initializer is LogicalExpression)
                            {
                                return SimpleType.Boolean;
                            }
                            else if (variable.Initializer is ArithmeticExpression)
                            {
                                return SimpleType.Number;
                            }
                            else
                            {
                                return SimpleType.String;
                            }
                        }

                    }

                }

                // add logic to get figure out based on the expression
            }
            return SimpleType.String;
            
        }
    }
}
