﻿//-----------------------------------------------------------------------
// <copyright file="XslReader.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace BtmToTrfmConvertor
{
    using System;
    using System.Collections.Generic;
    using System.Xml;
    using Logger;
    using Microsoft.ApplicationServer.Integration.Transforms.MapOM;
 
    class XslReader
    {
        private string localNameTarget;
        private bool conditionPresent;

        XmlReader reader = null;

        public XslReader(XmlReader xmlReader)
        {
            Mapeach = false;
            this.reader = xmlReader;
            this.localNameTarget = "localName";
            this.conditionPresent = false;
        }

        public bool Mapeach { get; private set; }

        // parses the xsl document till it reaches the target schema's subtree
        public void Parse()
        {
            // string localNameSource = string.Empty;
            OperationContext context = new OperationContext();
            List<object> statementList = new List<object>();
            string readerName;
            try
            {
                while (this.reader.Read())
                {
                    switch (this.reader.NodeType)
                    {
                        case XmlNodeType.Element:
                            // Get element Name
                            readerName = PathCreation.RemovePrefixFromNode(this.reader.Name);
                            switch (readerName)
                            {
                                case Constants.XslConstants.StylesheetTag:
                                    // adding all the namespaces to the dictionary
                                    // bool firstIteration = true;
                                    if (!this.reader.HasAttributes)
                                    {
                                        break;
                                    }

                                    this.reader.MoveToFirstAttribute();
                                    do
                                    {
                                        if (this.reader.Prefix.Equals("xmlns"))
                                        {
                                            string prefix = this.reader.Name.Substring(6);
                                            Constants.NamespacePrefix.Add(prefix, this.reader.Value);
                                        }
                                    } while (this.reader.MoveToNextAttribute());  

                                    break;
                                case Constants.XslConstants.VariableTag:
                                    // variables
                                    IXslElementHandler handler = new VariableHandler(this.reader.ReadSubtree(), context);
                                    handler.HandleElementConversion();
                                    break;
                                case Constants.XslConstants.TemplateTag:
                                    this.reader.MoveToAttribute(Constants.XslConstants.MatchAttribute);
                                    if (!this.reader.Value.Equals("/"))
                                    {
                                        int charCut = this.reader.Value.IndexOf(':');
                                        if (charCut == -1)
                                        {
                                            charCut++;
                                        }

                                        Constants.sourceTemplate = this.reader.Value;
                                        // localNameSource = this.reader.Value.Substring(charCut + 1);
                                        this.localNameTarget = string.Empty;
                                        this.Read(context);
                                        statementList = context.GetItemsInCurrentContext();
                                        Block mapletBlock = Constants.map1;
                                        foreach (object obj in statementList)
                                        {
                                            // gets the object type and adds it to the map
                                            Controller.log.trace(MsgType.INFO, "object of type:" + obj.GetType().ToString() + " is being added to the maplet");
                                            BlockCreator.AddToBlock(ref mapletBlock, obj);
                                        }

                                        Constants.map1 = (Maplet)mapletBlock;
                                    }

                                    break;
                                default:
                                    break;
                            }

                            break;
                        case XmlNodeType.EndElement:
                            break;
                        default:
                            break;
                    }
                }
            }
            catch (Exception e)
            {
                //Console.WriteLine(e.Message);
                //Console.WriteLine(e.StackTrace);
                Controller.log.debug(MsgType.ERROR, "Message:" + e.Message + "\nStackTrace:" + e.StackTrace);
            }

            // return localNameSource + "&" + this.localNameTarget;
        }

        // parses the tree using the current context
        public void Read(OperationContext context)
        {
            bool generateSubtreePath = false;
            bool conditionLoop = true;
            XslExpressionHandler expressionHandler = new XslExpressionHandler(context);
            IXslElementHandler handler;
            ExpressionArgument conditionValue = null;
            string value = string.Empty;
            string readerName;
            if (string.IsNullOrEmpty(context.SubTreePath))
            {
                context.SubTreePath = Constants.TRFMConstants.ScopeTargetHome;
            }
            try
            {
                while (this.reader.Read())
                {
                    object statement;
                    readerName = PathCreation.RemovePrefixFromNode(this.reader.Name);
                    switch (this.reader.NodeType)
                    {
                        case XmlNodeType.Element:
                            switch (readerName)
                            {
                                case Constants.XslConstants.AttributeTag:
                                    // if this is inside a loop then its a mapeach
                                    Mapeach = true;
                                    value = string.Empty;

                                    // attribute elements present in the tree
                                    this.reader.MoveToAttribute(Constants.XslConstants.NameAttribute);
                                    if (this.reader.Value.Trim().Equals("xsi:nil"))
                                    {
                                        Controller.log.trace(MsgType.INFO, "xsi:nil encountered");
                                        this.reader.Skip();
                                        break;
                                    }

                                    string target = PathCreation.CreateTargetNode(context.SubTreePath) + "/@" + this.reader.Value;

                                    while (!this.reader.Name.Equals(Constants.XslConstants.XslPrefix + ":" + Constants.XslConstants.ValueOfTag))
                                    {
                                        this.reader.Read();
                                    }

                                    this.reader.MoveToAttribute(Constants.XslConstants.SelectAttribute);
                                    value = this.reader.Value.Trim();
                                    object sourceExpression = expressionHandler.ParseExpression(value);
                                    PathExpression pathExpressionSource = (PathExpression)this.GenerateSourceExpressionArgumentBasedOnInput(sourceExpression, context, Constants.XslConstants.AttributeTag);

                                    if (pathExpressionSource != null)
                                    {
                                        if (this.conditionPresent)
                                        {
                                            handler = new ConditionalAssignmentHandler(conditionValue, pathExpressionSource, this.GenerateTargetExpressionArgument(target));
                                            statement = handler.HandleElementConversion();
                                            if (statement != null)
                                            {
                                                context.AddItemToCurrentContext(statement);
                                                Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Conditional Assignment created successfully");
                                            }
                                        }
                                        else
                                        {
                                            handler = new LinkHandler(pathExpressionSource, this.GenerateTargetExpressionArgument(target));
                                            context.AddItemToCurrentContext(handler.HandleElementConversion());
                                            Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Link created successfully");
                                        }
                                    }
                                    else
                                    {
                                        Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Link not created as source node is null");
                                    }

                                    break;

                                case Constants.XslConstants.IfTag:
                                    // Condition encountered
                                    this.reader.MoveToAttribute(Constants.XslConstants.TestAttribute);
                                    value = this.reader.Value;
                                    if (value[0] != '@')
                                    {
                                        // not an attribute
                                        this.conditionPresent = true;
                                        value = value.Trim();
                                        object condition = expressionHandler.ParseExpression(value);
                                        conditionValue = this.GenerateSourceExpressionArgumentBasedOnInput(condition, context, Constants.XslConstants.IfTag);

                                        if (conditionValue == null)
                                        {
                                            this.conditionPresent = false;
                                        }
                                        else if (conditionLoop)
                                        {
                                            Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Condition variable created for loop:" + conditionValue);
                                            context.AddItemToCurrentContext(conditionValue);
                                        }
                                    }

                                    break;

                                case Constants.XslConstants.ForeachTag:
                                    // looping structure
                                    // in case of foreach loop,reassign the values from the mapeach loop
                                    string mapeachSource, earlierScope, earlierSubtree, newLoopVariable;
                                    mapeachSource = earlierSubtree = earlierScope = newLoopVariable = string.Empty;
                                    bool changeToForeach = false;
                                    if (context.Scope != "Home" && !Mapeach)
                                    {
                                        this.ChangeFromForEachToMapeach(ref context, out mapeachSource, out earlierScope, out earlierSubtree, out newLoopVariable);
                                        changeToForeach = true;
                                    }

                                    handler = new ForEachHandler(this.reader.ReadSubtree(), context);
                                    statement = handler.HandleElementConversion();
                                    if (statement != null)
                                    {
                                        if (context.CurrentListInitialisation != null)
                                        {
                                            ListCreator listCreator = new ListCreator(context.CurrentListInitialisation, context);
                                            listCreator.AddObjectToList(statement);
                                            Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Loop created successfully and added to list");
                                        }
                                        else
                                        {
                                            context.AddItemToCurrentContext(statement);
                                            Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Loop created successfully");
                                        }
                                    }
                                    else
                                    {
                                        // reverting the above change in case a null value is returned.
                                        if (changeToForeach)
                                        {
                                            this.RevertChangeFromForEachToMapeach(ref context, mapeachSource, earlierScope, earlierSubtree, newLoopVariable);
                                        }
                                    }

                                    break;

                                case Constants.XslConstants.ValueOfTag:
                                    // if this is inside a loop then its a mapeach
                                    Mapeach = true;
                                    while (this.reader.MoveToNextAttribute())
                                    {
                                        if (this.reader.Name.Equals(Constants.XslConstants.SelectAttribute))
                                        {
                                            if (this.reader.Value != "./text()")
                                            {
                                                value = this.reader.Value.Trim();
                                                object sourceValue = expressionHandler.ParseExpression(value);
                                                ExpressionArgument sourceNode = this.GenerateSourceExpressionArgumentBasedOnInput(sourceValue, context, Constants.XslConstants.ValueOfTag);

                                                if (sourceNode != null)
                                                {
                                                    string temp = PathCreation.CreateTargetNode(context.SubTreePath);
                                                    PathExpression pathExpressionTarget = this.GenerateTargetExpressionArgument(temp);

                                                    if (this.conditionPresent)
                                                    {
                                                        handler = new ConditionalAssignmentHandler(conditionValue, sourceNode, pathExpressionTarget);
                                                        statement = handler.HandleElementConversion();
                                                        if (statement != null)
                                                        {
                                                            context.AddItemToCurrentContext(statement);
                                                            Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Conditional Assignment created successfully");
                                                        }
                                                    }
                                                    else
                                                    {
                                                        if (sourceNode is ConstantExpression)
                                                        {
                                                            VariableHandler varHandler = new VariableHandler(context);
                                                            string input = (string)((ConstantExpression)sourceNode).Value;
                                                            VariableInitialization constantVariable = varHandler.CreateConstantFunctoid(input, true);
                                                            context.AddItemToCurrentContext(constantVariable);
                                                            sourceNode = ExpressionArgumentCreator.CreatePathExpression(constantVariable.Identifier);
                                                        }

                                                        handler = new LinkHandler((PathExpression)sourceNode, pathExpressionTarget);
                                                        context.AddItemToCurrentContext(handler.HandleElementConversion());
                                                        Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Link created successfully");
                                                    }
                                                }
                                                else
                                                {
                                                    Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Error in creation of a conditional assignment or link between a functoid and a target node");
                                                }
                                            }

                                            break;
                                        }
                                    }

                                    break;

                                case Constants.XslConstants.VariableTag:
                                    // variables declared in xsl file
                                    handler = new VariableHandler(this.reader.ReadSubtree(), context);
                                    statement = handler.HandleElementConversion();
                                    if (statement != null)
                                    {
                                        context.AddItemToCurrentContext(statement);
                                        Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Variable created successfully");
                                    }

                                    break;

                                case Constants.XslConstants.CopyOfTag:
                                    Mapeach = true;
                                    VariableHandler massCopyHandler = new VariableHandler(context);
                                    VariableInitialization massCopyVariableNode = massCopyHandler.AddFunctoid("Mass copy functoid", "()", true);
                                    if (massCopyVariableNode != null)
                                    {
                                        context.AddItemToCurrentContext(massCopyVariableNode);
                                        Controller.log.trace(MsgType.INFO, "Mass copy functoid created");
                                    }
                                    else
                                    {
                                        Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Failure to create a dummy for mass copy in TRFM");
                                    }

                                    LinkHandler linkHandler = new LinkHandler(massCopyVariableNode.Identifier, PathCreation.CreateTargetNode(context.SubTreePath));
                                    statement = linkHandler.HandleElementConversion();
                                    if (statement != null)
                                    {
                                        context.AddItemToCurrentContext(statement);
                                        Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Link created successfully");
                                    }

                                    this.reader.Read();
                                    this.reader.Read();
                                    break;

                                case Constants.XslConstants.MessageTag:
                                    VariableHandler assertFunctoidHandler = new VariableHandler(context);
                                    this.reader.Read();
                                    this.reader.Read();
                                    VariableInitialization assertVariableNode = null;
                                    if (this.reader.Name.Equals(Constants.XslConstants.ValueOfTag))
                                    {
                                        this.reader.MoveToAttribute(Constants.XslConstants.SelectAttribute);
                                        assertVariableNode = assertFunctoidHandler.AddFunctoid("Assert functoid-false condition", PathCreation.CreateSourceNode(this.reader.Value, context), true);
                                    }

                                    if (assertVariableNode != null)
                                    {
                                        context.AddItemToCurrentContext(assertVariableNode);
                                        Controller.log.trace(MsgType.INFO, "Assert functoid created ");
                                    }
                                    else
                                    {
                                        Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "Failure to create a dummy for assert functoid in TRFM");
                                    }

                                    LinkHandler linkHandler1 = new LinkHandler(assertVariableNode.Identifier, PathCreation.CreateTargetNode(context.SubTreePath));
                                    statement = linkHandler1.HandleElementConversion();
                                    if (statement != null)
                                    {
                                        context.AddItemToCurrentContext(statement);
                                        Controller.log.trace(MsgType.SUCCESSFUL_CONVERSION, "Link created successfully");
                                    }

                                    break;

                                case Constants.XslConstants.TextTag:
                                    // when a value is given to a schema node
                                    string constant = this.reader.ReadInnerXml();
                                    if (!string.IsNullOrEmpty(constant))
                                    {
                                        VariableHandler constantFunctoidHandler = new VariableHandler(context);
                                        statement = constantFunctoidHandler.CreateConstantFunctoid(constant, true);
                                        context.AddItemToCurrentContext(statement);
                                        LinkHandler linkHandler2 = new LinkHandler(((VariableInitialization)statement).Identifier, PathCreation.CreateTargetNode(context.SubTreePath));
                                        statement = linkHandler2.HandleElementConversion();
                                        if (statement != null)
                                        {
                                            context.AddItemToCurrentContext(statement);
                                            Controller.log.trace(MsgType.INFO, "The value given to a source node is mapped to a string concate functoid successfully:" + constant);
                                        }
                                    }

                                    break;

                                default:
                                    if (!string.IsNullOrEmpty(readerName))
                                    {
                                        conditionLoop = false;
                                        if (string.IsNullOrEmpty(this.localNameTarget))
                                        {
                                            this.localNameTarget = readerName;
                                            generateSubtreePath = true;
                                        }
                                        else
                                        {
                                            if (!this.reader.Prefix.Equals("xsl") && !this.reader.IsEmptyElement)
                                            {
                                                if (generateSubtreePath)
                                                {
                                                    context.SubTreePath = context.SubTreePath + "/" + this.reader.Name;
                                                    Controller.log.trace(MsgType.INFO, "subtree path:" + context.SubTreePath);
                                                }
                                                else
                                                {
                                                    // in a new scope,generateSubtreePath is switched off. 
                                                    // This means scope might be a map each scope and the mapeachName is added to the dictionary in case of not being the home scope
                                                    if (context.Scope != "Home")
                                                    {
                                                        TrfmCounters.MapDictionary.Add(Constants.TRFMConstants.MapeachPrefix + TrfmCounters.MapEachCounter, this.reader.Name);
                                                    }

                                                    generateSubtreePath = true;
                                                }
                                            }
                                        }
                                    }

                                    break;
                            }

                            break;
                        case XmlNodeType.EndElement:
                            if (readerName.Equals(Constants.XslConstants.IfTag))
                            {
                                this.conditionPresent = false;
                            }
                            else
                            {
                                int val = context.SubTreePath.LastIndexOf('/');
                                if (this.reader.Name.Equals(context.SubTreePath.Substring(val + 1, context.SubTreePath.Length - val - 1)))
                                {
                                    context.SubTreePath = context.SubTreePath.Substring(0, val);
                                }
                            }

                            break;
                        default:
                            break;
                    }
                }
            }
            catch (Exception e)
            {
                Controller.log.debug(MsgType.ERROR, "Message:" + e.Message + "\nStackTrace:" + e.StackTrace);
                Controller.log.debug(MsgType.UNSUCCESSFUL_CONVERSION, "There was an Unexpected error around " + reader.Name);
            }
        }

        private void RevertChangeFromForEachToMapeach(ref OperationContext context, string mapeachSource, string earlierScope, string earlierSubtree, string newLoopVariable)
        {
            context.Scope = earlierScope;
            TrfmCounters.ForEachCounter--;
            TrfmCounters.MapEachCounter++;
            if (!string.IsNullOrEmpty(mapeachSource) && !TrfmCounters.MapDictionary.ContainsKey(Constants.TRFMConstants.MapeachPrefix + TrfmCounters.MapEachCounter))
            {
                TrfmCounters.MapDictionary.Add(Constants.TRFMConstants.MapeachPrefix + TrfmCounters.MapEachCounter, mapeachSource);
                context.SubTreePath = earlierSubtree;
            }

            context.ReplaceScopeAndLoopVariables(newLoopVariable, earlierScope.Substring(earlierScope.LastIndexOf('/') + 1));
            Controller.log.trace(MsgType.INFO, "scope changed back to :" + context.Scope);
        }

        private void ChangeFromForEachToMapeach(ref OperationContext context, out string mapeachSource, out string earlierScope, out string earlierSubtree, out string newLoopVariable)
        {
            TrfmCounters.ForEachCounter++;
            if (TrfmCounters.MapDictionary.ContainsKey(Constants.TRFMConstants.MapeachPrefix + TrfmCounters.MapEachCounter))
            {
                mapeachSource = TrfmCounters.MapDictionary[Constants.TRFMConstants.MapeachPrefix + TrfmCounters.MapEachCounter];
                TrfmCounters.MapDictionary.Remove(Constants.TRFMConstants.MapeachPrefix + TrfmCounters.MapEachCounter);
                earlierSubtree = context.SubTreePath;
                context.SubTreePath = context.SubTreePath + "/" + mapeachSource;
            }
            else
            {
                mapeachSource = earlierSubtree = string.Empty;
            }

            TrfmCounters.MapEachCounter--;
            earlierScope = context.Scope;
            newLoopVariable = Constants.TRFMConstants.ForeachPrefix + TrfmCounters.ForEachCounter;
            context.ReplaceScopeAndLoopVariables(context.Scope.Substring(context.Scope.LastIndexOf('/') + 1), newLoopVariable);
            context.Scope = context.Scope.Substring(0, context.Scope.LastIndexOf('/')) + "/" + newLoopVariable;
            Controller.log.trace(MsgType.INFO, "scope:" + context.Scope);
        }

        private PathExpression GenerateTargetExpressionArgument(string target)
        {
            if (target == null)
            {
                return null;
            }
            else
            {
                return ExpressionArgumentCreator.CreatePathExpression(target);
            }
        }

        // Gives an expressionargument mainly for the source path based on the input object created from the source node
        // input tag denotes the xsl operator for which this function is called
        private ExpressionArgument GenerateSourceExpressionArgumentBasedOnInput(object inputObject, OperationContext context, string inputTag)
        {
            ExpressionArgument expressionArgument = null;
            if (inputObject == null)
            {
                return null;
            }
            else if (inputObject.GetType() == typeof(ConstantExpression))
            {
                switch (inputTag)
                {
                    case Constants.XslConstants.AttributeTag:
                        Controller.log.trace(MsgType.INFO, "constant value encountered");
                        VariableHandler varHandler = new VariableHandler(context);
                        string input = (string)((ConstantExpression)inputObject).Value;
                        VariableInitialization constantVariable = varHandler.CreateConstantFunctoid(input, true);
                        context.AddItemToCurrentContext(constantVariable);
                        expressionArgument = ExpressionArgumentCreator.CreatePathExpression(constantVariable.Identifier);
                        break;
                    default:
                        expressionArgument = (ExpressionArgument)inputObject;
                        break;
                }
            }
            else if (inputObject.GetType() == typeof(PathExpression))
            {
                switch (inputTag)
                {
                    case Constants.XslConstants.IfTag:
                        if (((PathExpression)inputObject).Path.StartsWith(Constants.TRFMConstants.VariableTag))
                        {
                            expressionArgument = (ExpressionArgument)inputObject;
                        }

                        break;
                    default:
                        expressionArgument = (PathExpression)inputObject;
                        break;
                }
            }
            else if (inputObject.GetType() == typeof(VariableInitialization))
            {
                VariableInitialization variable = (VariableInitialization)inputObject;
                if (variable.Initializer is LogicalExpression)
                {
                    expressionArgument = Optimiser.redundantLogicalExpression(variable.Initializer as LogicalExpression, context);
                    if (expressionArgument == null)
                    {
                        context.AddItemToCurrentContext(inputObject);
                        expressionArgument = ExpressionArgumentCreator.CreatePathExpression(variable.Identifier);
                    }
                }
                else
                {
                    context.AddItemToCurrentContext(inputObject);
                    expressionArgument = ExpressionArgumentCreator.CreatePathExpression(variable.Identifier);
                }
            }

            return expressionArgument;
        }
    }
}
