﻿//-----------------------------------------------------------------------
// <copyright file="Log.cs" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace Logger
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.IO;
    using System.Linq;
    using System.Text;

    public class Log
    {
        private static string logPath = string.Empty;
        private static Log logger = null;
        TextWriterTraceListener debugListener;
        TextWriterTraceListener traceListener;

        private Log()
        { 
            this.debugListener = new TextWriterTraceListener(System.IO.File.CreateText("Log.txt"));
            this.traceListener = new TextWriterTraceListener("Log.txt");
            logPath = Path.GetFullPath("Log.txt");
            Debug.Listeners.Add(this.debugListener);
            Trace.Listeners.Add(this.traceListener);
        }

        public string LogPath
        {
            get { return Log.logPath; }
        }

        public static Log CreateInstance()
        {
            if (null == logger)
            {
                logger = new Log();
            }

            return logger;
        }

        // logs the input to the debug listener
        public void debug(string msgFormat, string message)
        {
            Debug.WriteLine("<" + msgFormat + ">");
            Debug.Indent();
            Debug.WriteLine(message);
            Debug.Flush();
            Debug.Unindent();
        }

        // creates a report of the successful and unsuccessful conversions
        public void CreateReport(string inputFile, string outputFile)
        {
            this.debugListener.Close();
            this.traceListener.Close();
            StreamWriter outFile = new StreamWriter(outputFile);
            StreamReader inFile = new StreamReader(inputFile);
            bool flag = false;
            int success = 0;
            int failure = 0;
            List<string> lines = new List<string>();
            string line;
            line = inFile.ReadLine();
            outFile.WriteLine("--------------------------------<SUMMARY>------------------------------------" + Environment.NewLine);
            while (line != null)
            {
                if (line.Trim().StartsWith("<" + MsgType.SUCCESSFUL_CONVERSION + ">"))
                {
                    success++;
                    flag = false;
                }
                else if (line.Trim().StartsWith("<" + MsgType.UNSUCCESSFUL_CONVERSION + ">"))
                {
                    failure++;
                    flag = true;
                }
                else if (line.Trim().StartsWith("<"))
                {
                    flag = false;
                }

                if (flag == true)
                {
                    lines.Add(line);
                }

                line = inFile.ReadLine();
            }

            outFile.WriteLine("Unsuccessful Conversion=" + failure);
            outFile.WriteLine("----------------------------------------------------------------------------");
            outFile.WriteLine(Environment.NewLine + Environment.NewLine + Environment.NewLine);
            outFile.WriteLine("----------------------------UNSUCCESSFUL CONVERSION--------------------------" + Environment.NewLine);
            foreach (string s in lines)
            {
                outFile.WriteLine(s);
            }
         
            outFile.Close();
        }

        // logs the input to the trace listener
        public void trace(string msgFormat, string message)
        {
            Trace.WriteLine("<" + msgFormat + ">");
            Trace.Indent();
            Trace.WriteLine(message);
            Trace.Flush();
            Trace.Unindent();
        }
    }
}
